use crossterm::event::{KeyCode, KeyEvent, KeyEventKind};
use color_eyre::Result;
use ratatui::{
    layout::{Alignment, Constraint, Flex, Layout, Rect}, 
    widgets::{Block, Clear, Paragraph}, 
    Frame
};
//use tokio::sync::mpsc::UnboundedSender;
use std::sync::mpsc::Sender;

use super::{Component, PartialComponent, CurrentTab};
use crate::{
    action::Action, 
    config::Config, 
    palette::StylePalette, 
    //types::{ActionLevel, ActionTarget}, 
};

#[derive(Debug)]
pub struct RotatePopup {
    is_active: bool,
    //action_tx: Option<UnboundedSender<Action>>,
    network_tx: Option<Sender<Action>>,
    palette: StylePalette
}

impl Default for RotatePopup {
    fn default() -> Self {
        Self::new()
    }
}

impl RotatePopup {
    pub fn new() -> Self {
        Self {
            is_active: false,
            //action_tx: None,
            network_tx: None,
            palette: StylePalette::default(),
        }
    }

    fn rotate_keys(&mut self) {
        todo!();
        //if let Some(network_tx) = &self.network_tx {
        //    let _ = network_tx.send(Action::RotateSessionKeys);
        //}
    }
}

impl PartialComponent for RotatePopup {
    fn set_active(&mut self, current_tab: CurrentTab) {
        match current_tab {
            CurrentTab::RotatePopup => self.is_active = true,
            _ => self.is_active = false,
        };
    }
}

impl Component for RotatePopup {
    fn register_network_handler(&mut self, tx: Sender<Action>) -> Result<()> {
        self.network_tx = Some(tx);
        Ok(())
    }

    fn register_config_handler(&mut self, config: Config) -> Result<()> {
        if let Some(style) = config.styles.get(&crate::app::Mode::Wallet) {
            self.palette.with_normal_style(style.get("normal_style").copied());
            self.palette.with_normal_border_style(style.get("normal_border_style").copied());
            self.palette.with_normal_title_style(style.get("normal_title_style").copied());
            self.palette.with_popup_style(style.get("popup_style").copied());
            self.palette.with_popup_title_style(style.get("popup_title_style").copied());
        }
        Ok(())
    }
    fn handle_key_event(&mut self, key: KeyEvent) -> Result<Option<Action>> {
        if self.is_active && key.kind == KeyEventKind::Press {
            match key.code {
                KeyCode::Enter => self.rotate_keys(),
                KeyCode::Esc => self.is_active = false, 
                _ => {},
            };
        }
        Ok(None)
    }

    fn draw(&mut self, frame: &mut Frame, area: Rect) -> Result<()> {
        if self.is_active {
            let (border_style, border_type) = self.palette.create_popup_style();
            let popup = Paragraph::new(" Do you want to proceed key rotation?")
                .block(Block::bordered()
                    .border_style(border_style)
                    .border_type(border_type)
                    .title_style(self.palette.create_popup_title_style())
                    .title_alignment(Alignment::Right)
                    .title("Enter to proceed / Esc to close"));
            let v = Layout::vertical([Constraint::Max(3)]).flex(Flex::Center);
            let h = Layout::horizontal([Constraint::Max(50)]).flex(Flex::Center);
            let [area] = v.areas(area);
            let [area] = h.areas(area);

            frame.render_widget(Clear, area);
            frame.render_widget(popup, area);
        }
        Ok(())
    }
}
